package de.intevation.db;

import java.util.Properties;

import java.io.IOException;
import java.io.InputStream;

import org.apache.log4j.Logger;

import java.util.Map;
import java.util.HashMap;
import java.util.Enumeration;

public class Statements
{
    private static Logger log = Logger.getLogger(Statements.class);

    public static final String RESOURCE_PATH = "/sql/";
    public static final String COMMON_PROPERTIES = "-common.properties";

    protected String type;
    protected String driver;

    protected Map<String, SymbolicStatement> statements;

    public Statements(String type, String driver) {
        this.type   = type;
        this.driver = driver;
    }

    public SymbolicStatement getStatement(String key) {
        return getStatements().get(key);
    }

    public Map<String, SymbolicStatement> getStatements() {
        if (statements == null) {
            statements = loadStatements();
        }
        return statements;
    }

    protected Map<String, SymbolicStatement> loadStatements() {
        Map<String, SymbolicStatement> statements =
            new HashMap<String, SymbolicStatement>();

        Properties properties = loadProperties();

        for (Enumeration e = properties.propertyNames(); e.hasMoreElements();) {
            String key = (String)e.nextElement();
            String value = properties.getProperty(key);
            SymbolicStatement symbolic = new SymbolicStatement(value);
            statements.put(key, symbolic);
        }

        return statements;
    }

    protected String driverToProperties() {
        return
            type + "-" + 
            driver.replace('.', '-').toLowerCase() + ".properties";
    }

    protected Properties loadCommon() {
        Properties common = new Properties();

        String path = RESOURCE_PATH + type + COMMON_PROPERTIES;

        InputStream in = Statements.class.getResourceAsStream(path);

        if (in != null) {
            try {
                common.load(in);
            }
            catch (IOException ioe) {
                log.error("cannot load defaults: " + path, ioe);
            }
            finally {
                try {
                    in.close();
                }
                catch (IOException ioe) {
                }
            }
        }
        else {
            log.warn("cannot find: " + path);
        }

        return common;
    }

    protected Properties loadProperties() {

        Properties common = loadCommon();

        Properties properties = new Properties(common);

        String path = RESOURCE_PATH + driverToProperties();

        InputStream in = Statements.class.getResourceAsStream(path);

        if (in != null) {
            try {
                properties.load(in);
            }
            catch (IOException ioe) {
                log.error("cannot load statements: " + path, ioe);
            }
            finally {
                try {
                    in.close();
                }
                catch (IOException ioe) {
                }
            }
        }
        else {
            log.warn("cannot find: " + path);
        }

        return properties;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
